#include "StdInclude.h"
#include "Globals.h"
#include "Tetris.h"
#include <stdlib.h>
#include <time.h>

#define XRES 1024
#define YRES 768

// TET_WIDTH // Size of the board
#define GRFX_WIDTH 24 // Size of the graphical squares
#define GRFX_HEIGHT 24

#define X_OFFS (8)
#define X_STEP (256) // Difference between each player's board
#define Y_OFFS ((YRES - GRFX_HEIGHT*TET_HEIGHT) / 2)

#define SCORE_X 8 // Position of the score display
#define SCORE_Y (NEXT_Y + 4*GRFX_HEIGHT+4)

#define NEXT_X (X_OFFS + 2*GRFX_WIDTH) // Position of the next piece display
#define NEXT_Y (4)

#define SAVED_X (X_OFFS + 4*GRFX_WIDTH) // Position of the next piece display
#define SAVED_Y (YRES - 16 - 4*GRFX_HEIGHT)

// ------------------------------------------------------------------
// cdx_Init - handles initialization of the CDX objects
// ------------------------------------------------------------------
BOOL cdx_Init()
{
	// Create the CDXSreen object
	Screen = new CDXScreen();
	if (Screen==NULL)
        CDXError( NULL , "Could not create CDXScreen object" );

	// start app in a window
	if (FAILED(Screen->CreateWindowed(g_hWnd, XRES, YRES)))
        CDXError( Screen , "Could not set windowed mode %dx%d", XRES, YRES );

	// create our input object
	Input = new CDXInput( );
	if( Input  ==  NULL ) 
        CDXError( Screen , "Could not create CDXInput object" );

	// Create input devices
	if(FAILED(Input->Create( g_hInst , g_hWnd)))
        CDXError( Screen , "Could not create direct input object" );
	if(FAILED(Input->SetActiveDevices(FALSE, TRUE, TRUE)))
        CDXError( Screen , "Could not acquire keyboard and joystick" );
	Input->SetJoystickExclusive(0, TRUE);


	BG = new CDXSurface();
	if (BG == NULL) 
		CDXError( Screen, "Could not create BG Surface");
	if (FAILED(BG->Create(Screen, "BG2.BMP", (BYTE)CDXMEM_VIDTHENSYS)))
		CDXError( Screen, "Could not load BG2.BMP");

	Tiles = new CDXTile();
	if (Tiles == NULL) 
		CDXError( Screen, "Could not create Tiles Surface");
	if (FAILED(Tiles->Create(Screen, "TILES.BMP", GRFX_WIDTH, GRFX_HEIGHT, 11, (BYTE)CDXMEM_VIDTHENSYS)))
		CDXError( Screen, "Could not load TILES.BMP");

	ScoreFont = new CDXBitmapFont();
	if (FAILED(ScoreFont->CreateFromFile(Screen, "Font010.bmp", 16, 16)))
		CDXError( Screen, "Cound not create font");

	for (int i=0; i<NUM_PLAYERS; i++) {
		Tetris[i] = new CTetris();
	}

	// Seed the random number generator
	srand( (unsigned)time( NULL ) );

	tmr.Start();
	
	return TRUE;
}



// ------------------------------------------------------------------
// cdx_DeInit - handles cleanup of CDX objects
// ------------------------------------------------------------------
void cdx_DeInit( void )
{
	// TODO: Destroy your CDX objects here

	SAFEDELETE( Input );	
	SAFEDELETE( Tiles );
	SAFEDELETE( ScoreFont );
	SAFEDELETE( BG );
	SAFEDELETE( Screen );
	for (int i=0; i<NUM_PLAYERS; i++) {
		SAFEDELETE( Tetris[i] );
	}
}




// ------------------------------------------------------------------
// AdjustWinStyle - adjusts the window style according to the mode
// ------------------------------------------------------------------
void AdjustWinStyle()
{
	RECT        rect = {0, 0, XRES, YRES};
	DWORD       dwStyle;

	// Change window attributes

	dwStyle = GetWindowStyle(g_hWnd);
	dwStyle &= ~WS_POPUP;
	dwStyle |= WS_OVERLAPPED | WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU;

	SetWindowLong(g_hWnd, GWL_STYLE, dwStyle);

	// Resize the window so that the client area is 640x480

	AdjustWindowRectEx(&rect, GetWindowStyle(g_hWnd), GetMenu(g_hWnd) != NULL,
		GetWindowExStyle(g_hWnd));

	// Just in case the window was moved off the visible area of the
	// screen.

	SetWindowPos(g_hWnd, NULL, 0, 0, rect.right-rect.left,
		rect.bottom-rect.top, SWP_NOMOVE | SWP_NOZORDER |
		SWP_NOACTIVATE);

	SetWindowPos(g_hWnd, HWND_NOTOPMOST, 0, 0, 0, 0,
		SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
}







// ------------------------------------------------------------------
// cdx_DoFrame - performs drawing of the current frame
// ------------------------------------------------------------------
void cdx_DoFrame()
{
	char buf[255];
	// clear the current frame
	BG->DrawBlk(Screen->GetBack(), 0, 0);

	for (int iPlayer=0; iPlayer<NUM_PLAYERS; iPlayer++) {
		board_t *board= Tetris[iPlayer]->getDrawnBoard();

		int x, y;
		LOOPBOARD(x, y) {
			if ((*board)[x][y]) {
				Tiles->Draw(x*GRFX_WIDTH+X_OFFS+X_STEP*iPlayer, y*GRFX_HEIGHT+Y_OFFS, Screen->GetBack(), CDXBLT_BLK, (*board)[x][y]);
			}
		}

		piece_t *piece = Tetris[iPlayer]->getNextPiece();
		for (x=0; x<P_WIDTH; x++)
			for (y=0; y<P_HEIGHT; y++) {
				if ((*piece)[x][y]) {
					Tiles->Draw(x*GRFX_WIDTH+NEXT_X+X_STEP*iPlayer, y*GRFX_HEIGHT+NEXT_Y, Screen->GetBack(), CDXBLT_BLK, (*piece)[x][y]);
				}
			}

		piece = Tetris[iPlayer]->getSavedPiece();
		for (x=0; x<P_WIDTH; x++)
			for (y=0; y<P_HEIGHT; y++) {
				if ((*piece)[x][y]) {
					Tiles->Draw(x*GRFX_WIDTH+SAVED_X+X_STEP*iPlayer, y*GRFX_HEIGHT+SAVED_Y, Screen->GetBack(), CDXBLT_BLK, (*piece)[x][y]);
				}
			}

		sprintf(buf, "SCORE: %d", Tetris[iPlayer]->iScore);
		ScoreFont->DrawTrans(SCORE_X+X_STEP*iPlayer, SCORE_Y, buf, Screen->GetBack());
//		Screen->GetBack()->TextXY(SCORE_X+X_STEP*iPlayer, SCORE_Y, RGB(255, 255, 255), buf);
		sprintf(buf, "HIGH SCORE: %d", g_iHighScore);
//		Screen->GetBack()->TextXY(SCORE_X+X_STEP*iPlayer, SCORE_Y+12, RGB(255, 255, 255), buf);
		ScoreFont->DrawTrans(SCORE_X+X_STEP*iPlayer, SCORE_Y+16, buf, Screen->GetBack());
//Debug		LONG jx, jy;
//		Input->GetJoystickDeltas(&jx, &jy);
//		sprintf(buf, "JoyPos: %d,%d", jx, jy);
//		Screen->GetBack()->TextXY(SCORE_X+X_STEP*iPlayer, SCORE_Y+24, RGB(255, 255, 255), buf);
	}

	Screen->Flip();	 // Flip the back buffer to the front
}




