// Tetris.h: interface for the CTetris class.
// A device/graphic library independent Tetris class
// DO NOT add any graphics-specific (i.e. CDX) code in this class
// Written by Jimb Esser
// Original Tetris game concept Alexy Pajitnov
// You must have <windows.h> included to include this file
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TETRIS_H__5C3E23D0_3F8A_11D4_BFF9_00009479D5D0__INCLUDED_)
#define AFX_TETRIS_H__5C3E23D0_3F8A_11D4_BFF9_00009479D5D0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PieceLib.h"

#define TET_WIDTH 10 // The board dimensions
#define TET_HEIGHT 20
#define P_WIDTH 4 // The pieces dimensions
#define P_HEIGHT 4


#define NUMPEICES	7

#define TYPE_GOLD 8
#define TYPE_SILVER 9

#define MS_PER_DROP 1000

typedef long long_board_t[TET_WIDTH][TET_HEIGHT];
typedef int board_t[TET_WIDTH][TET_HEIGHT];
typedef int piece_t[P_WIDTH][P_HEIGHT];

#define LOOPBOARD(x, y) for (y=0; y<TET_HEIGHT; y++) for (x=0; x<TET_WIDTH; x++)

class CTetris  
{
public:
	void Update(DWORD dwTime); // Time in MS
	void CheckMadeLine();
	void Reset();
	board_t *getDrawnBoard();
	piece_t *getNextPiece() { return &nextpiece; }
	piece_t *getSavedPiece() { return &savedpiece; }
	void RotateCCW();
	void RotateCW();
	void goDrop();
	void goLeft();
	void goRight();
	void goDown();
	void swapToSaved();
	CTetris();
	~CTetris();

	bool bDead;
	int iScore;
	int iLines;

private:
	void NewPiece();
	void SetPiece();
	bool Clash();
	void CheckGoldSilver();

	long_board_t board; // internal representation
	piece_t piece;
	piece_t nextpiece;
	piece_t savedpiece; // The saved piece
	board_t drawnBoard; // board to be drawn

	CPieceLib *pieceLib; // Used for remembering what pieces have been drawn

	int px, py; // Locations of the upper-left corner of the piece;
	long liNextPieceID;  // ID of the next piece
	long liSavedPieceID; // ID of the saved piece
	long liPieceID; // ID of the current piece (from the pieceLib)

	bool bJustSwapped;
	int iSeed;

	DWORD dwLastAction;
};


#endif // !defined(AFX_TETRIS_H__5C3E23D0_3F8A_11D4_BFF9_00009479D5D0__INCLUDED_)
