#include "StdInclude.h"
#include "Globals.h"
#include "graphics.h"
#include "Tetris.h"
#include "UI.h"


enum
{
	KEY_JOY0LEFT				= CDXKEY_NUMKEYS,
	KEY_JOY0RIGHT,
	KEY_JOY0UP,
	KEY_JOY0DOWN,

	KEY_JOY1LEFT,
	KEY_JOY1RIGHT,
	KEY_JOY1UP,
	KEY_JOY1DOWN
} EXTRA_KEY_ENUM;


// Function Table Defines
#define MAX_UI_FUNCS 100
typedef void (*VOIDINTFUNC)(int);
typedef struct _FuncTableEntry {
	WORD wKey;
	BYTE bState;
	VOIDINTFUNC func;
	int iParam; // passed to func
} FuncTableEntry;
// bState can be CDXKEY_NONE, CDXKEY_RELEASE, CDXKEY_PRESS, or CDXKEY_REPEAT

// Globals
FuncTableEntry UIFuncTable[MAX_UI_FUNCS];
int iNumFuncs=0;
bool bUIInited=false; // Whether or not the UI func table is inited


void ClearFuncTable(void)
{
	for (int i=0; i<MAX_UI_FUNCS; i++)
	{
		UIFuncTable[i].bState=0;
		UIFuncTable[i].wKey=0;
		UIFuncTable[i].func=NULL;
		UIFuncTable[i].iParam=-1;
	}
	iNumFuncs = 0;
	bUIInited=false;
}

void AddFunc(WORD wKey, BYTE bState, VOIDINTFUNC func, int iParam)
{
	if (iNumFuncs<MAX_UI_FUNCS)
	{
		UIFuncTable[iNumFuncs].bState=bState;
		UIFuncTable[iNumFuncs].wKey=wKey;
		UIFuncTable[iNumFuncs].func=func;
		UIFuncTable[iNumFuncs].iParam=iParam;
		iNumFuncs++;
	}
}


// Looks through everything in the UI Funciton Table and checks to see if any
//  of the conditions are met to call the UI functions
void HandleUIGeneric(void)
{
	static long jx[4]={0}, jy[4]={0}, ljx[4]={0}, ljy[4]={0}; // Joy positions
	Input->Update();
	int i;
	for (i=0; i<2; i++) {
		ljx[i] = jx[i];
		ljy[i] = jy[i];
		Input->GetJoystickDeltas(i, &jx[i], &jy[i]);
	}

	for (i=0; i<iNumFuncs; i++) {
		if (UIFuncTable[i].wKey>=CDXKEY_NUMKEYS) {
			// Special
			int iJoyNum=0;
			WORD w = UIFuncTable[i].wKey;
			while (w > KEY_JOY0DOWN) {
				iJoyNum++;
				w-=4;
			}
			long x1, x2, y1, y2;
			switch (UIFuncTable[i].bState) {
			case CDXKEY_PRESS:
				x1=ljx[iJoyNum]; x2=jx[iJoyNum];
				y1=ljy[iJoyNum]; y2=jy[iJoyNum];
				break;
			case CDXKEY_RELEASE:
				x1=jx[iJoyNum]; x2=ljx[iJoyNum];
				y1=jy[iJoyNum]; y2=ljy[iJoyNum];
				break;
			}
			bool b=false;
			switch (w) {
			case KEY_JOY0RIGHT:
				b=x1<500&&x2>500;
				break;
			case KEY_JOY0LEFT:
				b=x1>-500&&x2<-500;
				break;
			case KEY_JOY0UP:
				b=y1>-500&&y2<-500;
				break;
			case KEY_JOY0DOWN:
				b=y1<500&&y2>500;
				break;

			}
			if (b) {
				UIFuncTable[i].func(UIFuncTable[i].iParam);
			}
		} else { // Normal
			if (Input->GetKeyState(UIFuncTable[i].wKey) == UIFuncTable[i].bState)
				UIFuncTable[i].func(UIFuncTable[i].iParam);
		}
	}
}


// Called to exit the game
void ExitFunc(int junk)
{
	g_iGameState = STATE_DEINIT;
}


// Called by the main loop to handle User Input
void doUI(void)
{
	if (!bUIInited)
		InitUI();

	HandleUIGeneric();
}

		
void goLeft(int iPlayer) { Tetris[iPlayer]->goLeft(); }
void goRight(int iPlayer) { Tetris[iPlayer]->goRight(); }
void goDown(int iPlayer) { Tetris[iPlayer]->goDown(); }
void goDrop(int iPlayer) { Tetris[iPlayer]->goDrop(); }
void rotCW(int iPlayer) { Tetris[iPlayer]->RotateCW(); }
void rotCCW(int iPlayer) { Tetris[iPlayer]->RotateCCW(); }
void swap(int iPlayer) { Tetris[iPlayer]->swapToSaved(); }



void InitUI(void)
{
	// This sets up a mapping between keypresses/releases and actions to perform
	AddFunc(CDXKEY_ESCAPE, CDXKEY_RELEASE, ExitFunc, 1);
	
	//  Player 0
	AddFunc(CDXKEY_A, CDXKEY_PRESS, goLeft, 0);
	AddFunc(CDXKEY_D, CDXKEY_PRESS, goRight, 0);
	AddFunc(CDXKEY_S, CDXKEY_PRESS, goDown, 0);
	AddFunc(CDXKEY_X, CDXKEY_PRESS, goDrop, 0);
	AddFunc(CDXKEY_W, CDXKEY_PRESS, rotCW, 0);
	AddFunc(CDXKEY_E, CDXKEY_PRESS, rotCCW, 0);
	AddFunc(CDXKEY_Q, CDXKEY_PRESS, swap, 0);

	// Player 1
	AddFunc(CDXKEY_LEFTARROW, CDXKEY_PRESS, goLeft, 1);
	AddFunc(CDXKEY_RIGHTARROW, CDXKEY_PRESS, goRight, 1);
	AddFunc(CDXKEY_DOWNARROW, CDXKEY_PRESS, goDown, 1);
	AddFunc(CDXKEY_RIGHTCTRL, CDXKEY_PRESS, goDrop, 1);
	AddFunc(CDXKEY_UPARROW, CDXKEY_PRESS, rotCW, 1);
	AddFunc(CDXKEY_BACKSLASH, CDXKEY_PRESS, rotCCW, 1);
	AddFunc(CDXKEY_NUM0, CDXKEY_PRESS, swap, 1);

	// Player 2
	AddFunc(CDXKEY_JOYBUTN1, CDXKEY_PRESS, goDrop, 2);
	AddFunc(CDXKEY_JOYBUTN0, CDXKEY_PRESS, rotCW, 2);
	AddFunc(CDXKEY_JOYBUTN2, CDXKEY_PRESS, rotCCW, 2);
	AddFunc(CDXKEY_JOYBUTN3, CDXKEY_PRESS, swap, 2);
	AddFunc(KEY_JOY0RIGHT, CDXKEY_PRESS, goRight, 2);
	AddFunc(KEY_JOY0LEFT, CDXKEY_PRESS, goLeft, 2);
	AddFunc(KEY_JOY0DOWN, CDXKEY_PRESS, goDown, 2);

	// Player 3
	AddFunc(CDXKEY_JOY1BUTN1, CDXKEY_PRESS, goDrop, 3);
	AddFunc(CDXKEY_JOY1BUTN0, CDXKEY_PRESS, rotCW, 3);
	AddFunc(CDXKEY_JOY1BUTN2, CDXKEY_PRESS, rotCCW, 3);
	AddFunc(CDXKEY_JOY1BUTN3, CDXKEY_PRESS, swap, 3);
	AddFunc(KEY_JOY1RIGHT, CDXKEY_PRESS, goRight, 3);
	AddFunc(KEY_JOY1LEFT, CDXKEY_PRESS, goLeft, 3);
	AddFunc(KEY_JOY1DOWN, CDXKEY_PRESS, goDown, 3);


	bUIInited = true;
}
